@@ -1751,6 +1751,18 @@ external_validation = function(true_labels, clusters, method = "adjusted_rand_in
   if (length(true_labels) != length(clusters)) stop('the length of the true_labels vector should equal the length of the clusters vector')
   if (!method %in% c('rand_index', 'adjusted_rand_index', 'jaccard_index', 'fowlkes_mallows_index', 'mirkin_metric', 'purity', 'entropy', 'nmi', 'var_info', 'nvi'))
     stop("supported methods are 'rand_index', 'adjusted_rand_index', 'jaccard_index', 'fowlkes_mallows_index', 'mirkin_metric', 'purity', 'entropy', 'nmi', 'var_info', 'nvi'")
+  
+  # if (method == 'nmi' || method == 'nvi') {
+  #   
+  #   unq_true = unique(true_labels)
+  #   
+  #   unq_clust = unique(clusters)
+  #   
+  #   if (length(unq_true) == 1 && length(unq_clust) == 1) {                # account for the case where true-labels and clusters perfectly match, SEE comments of issue https://github.com/mlampros/ClusterR/issues/8, https://github.com/scikit-learn/scikit-learn/blob/a24c8b46/sklearn/metrics/cluster/supervised.py#L772
+  #     
+  #     return(1.0)
+  #   }
+  # }
 
   tbl = table(clusters, true_labels)
 
@@ -1768,8 +1780,6 @@ external_validation = function(true_labels, clusters, method = "adjusted_rand_in
   #   }
   # }
 
-  conv_df = as.data.frame.matrix(tbl)
-
   tp_plus_fp = sum(gmp::asNumeric(gmp::chooseZ(rowSums(conv_df), 2)))
 
   tp_plus_fn = sum(gmp::asNumeric(gmp::chooseZ(colSums(conv_df), 2)))
@@ -1804,33 +1814,52 @@ external_validation = function(true_labels, clusters, method = "adjusted_rand_in
   }
 
   if (summary_stats || method == 'nmi' || method == 'var_info' || method == 'nvi') {
-
+    
     mutual_information = 0.0
-
+      
     joint_entropy = 0.0
-
+    
     for (i in 1:nrow(conv_df)) {
-
+      
       for (j in 1:ncol(conv_df)) {
-
+        
         if (conv_df[i,j] > 0.0) {
-
+          
           joint_entropy = joint_entropy + (-((conv_df[i,j] / sum(tbl)) * log2(conv_df[i,j] / sum(tbl))))
-
-          mutual_information = mutual_information + ((conv_df[i,j] / sum(tbl)) * log2((sum(tbl) * conv_df[i,j]) / (sum(conv_df[i,]) * sum(conv_df[,j]))))
+          
+          # mutual_information = mutual_information + ((conv_df[i,j] / sum(tbl)) * log2((sum(tbl) * conv_df[i,j]) / (sum(conv_df[i,]) * sum(conv_df[,j]))))       # SEE the comments of issue https://github.com/mlampros/ClusterR/issues/8 
+          
+          mutual_information = mutual_information + ((conv_df[i,j] / sum(tbl)) * log2(as.numeric(gmp::as.bigz(as.numeric(sum(tbl)) * as.numeric(conv_df[i,j])) / gmp::as.bigz(as.numeric(sum(conv_df[i,])) * as.numeric(sum(conv_df[,j]))))))
         }
       }
     }
-
+    
     entr_cluster = sum(apply(conv_df, 1, function(x) -(sum(x) / sum(tbl)) * log2(sum(x) / sum(tbl))))
-
+    
     entr_class = sum(apply(conv_df, 2, function(x) -(sum(x) / sum(tbl)) * log2(sum(x) / sum(tbl))))
-
-    NMI = (mutual_information / ((entr_cluster + entr_class) / 2.0))
-
+    
+    if (summary_stats || method == 'nmi' || method == 'nvi') {
+      
+      unq_true = unique(true_labels)
+      
+      unq_clust = unique(clusters)
+      
+      if (length(unq_true) == 1 && length(unq_clust) == 1) {                # account for the case where true-labels and clusters perfectly match, SEE comments of issue https://github.com/mlampros/ClusterR/issues/8, https://github.com/scikit-learn/scikit-learn/blob/a24c8b46/sklearn/metrics/cluster/supervised.py#L772
+        
+        NMI = 1.0
+        
+        NVI = 1.0
+      }
+      
+      else {
+        
+        NMI = (mutual_information / ((entr_cluster + entr_class) / 2.0))
+        
+        NVI = 1.0 - (mutual_information / joint_entropy)
+      }
+    }
+    
     VAR_INFO = (entr_cluster + entr_class) - 2.0 * mutual_information
-
-    NVI = 1.0 - (mutual_information / joint_entropy)
   }
 
   if (summary_stats) {
